﻿using Prism.Ioc;
using Borepin.PageModel;
using Borepin.Page;
using Xamarin.Forms;
using Borepin.Dialog;
using Borepin.DialogModel;
using Borepin.Service.Connections;
using Borepin.Service.BFFH;
using Borepin.Service.Credentials;

namespace Borepin
{
    public partial class App
    {
        public App()
        {

        }

        protected override async void OnInitialized()
        {
            InitializeComponent();

            var result = await NavigationService.NavigateAsync("/MainPage/NavigationPage/ServerListPage");

            if (!result.Success)
            {
                System.Diagnostics.Debugger.Break();
            }
        }

        protected override void RegisterTypes(IContainerRegistry containerRegistry)
        {
            // Register Navigation
            containerRegistry.RegisterForNavigation<NavigationPage>();
            containerRegistry.RegisterForNavigation<MainPage, MainPagePageModel>();
            containerRegistry.RegisterForNavigation<MachinesPage, MachinesPageModel>();
            containerRegistry.RegisterForNavigation<SettingsPage>();
            containerRegistry.RegisterForNavigation<MachinePage, MachinePageModel>();
            containerRegistry.RegisterForNavigation<LoginPasswordPage, LoginPasswordPageModel>();
            containerRegistry.RegisterForNavigation<HostSelectPage, HostSelectPageModel>();
            containerRegistry.RegisterForNavigation<LoginChoosePage, LoginChoosePageModel>();
            containerRegistry.RegisterForNavigation<ServerListPage, ServerListPageModel>();
            containerRegistry.RegisterForNavigation<ServerPage, ServerPageModel>();
            containerRegistry.RegisterForNavigation<ListPage, ListPageModel>();

            // Register Dialog
            containerRegistry.RegisterDialog<ConfirmDialog, ConfirmDialogModel>();

            // Register Service
            IConnectionService connectionService = new ConnectionService();
            containerRegistry.RegisterInstance<IConnectionService>(connectionService);
            ICredentialService credentialService = new CredentialService();
            containerRegistry.RegisterInstance<ICredentialService>(credentialService);

            containerRegistry.RegisterInstance<IBFFHService>(new BFFHService(connectionService, credentialService));
            containerRegistry.RegisterInstance<IConnectionService>(new ConnectionService());
        }
    }
}
