﻿using Borepin.Model;
using Borepin.Service.Connections;
using Borepin.Service.Credentials;
using System.Threading.Tasks;
using Capnp.Rpc;
using System.Collections.Generic;

namespace Borepin.Service.BFFH
{
    public class BFFHService : IBFFHService
    {
        private IConnectionService _ConnectionService;
        private ICredentialService _CredentialService;

        private FabAccessAPI.Connection _Connection;

        public BFFHService(IConnectionService connectionService, ICredentialService credentialService)
        {
            _ConnectionService = connectionService;
            _CredentialService = credentialService;
        }

        public Model.Connection ActiveConnection { get; private set; }

        public bool IsAuthenticated { get; private set; }

        public Task<bool> Connect(Model.Connection connection)
        {
            if (_Connection != null || ActiveConnection != null)
            {
                throw new System.Exception("Still connected");
            }

            var rpcClient = new TcpRpcClient();

            rpcClient.Connect(connection.Address.Host, connection.Address.Port);

            FabAccessAPI.Connection connection_test = new FabAccessAPI.Connection(rpcClient);

            _Connection = connection_test;
            ActiveConnection = connection;

            return Task.FromResult(true);
        }

        public Task<bool> Disconnect()
        {
            _Connection.RpcClient?.Dispose();
            _Connection = null;
            ActiveConnection = null;

            return Task.FromResult(true);
        }

        public bool CanAuthenticate()
        {
            return ActiveConnection.AuthenticationTyp == AuthenticationTyp.PLAIN && ActiveConnection.Username != string.Empty;
        }

        public async Task<bool> Authenticate()
        {
            string password = await _CredentialService.GetPasswordAsync(ActiveConnection);

            await _Connection.Auth("PLAIN", new Dictionary<string, object> { { "Username", ActiveConnection.Username }, { "Password", password } });

            return await Task.FromResult(true);
        }

        public async Task<bool> Authenticate(Model.Connection connection, string password)
        {
            await _Connection.Auth("PLAIN", new Dictionary<string, object> { { "Username", ActiveConnection.Username }, { "Password", password } });

            await _CredentialService.AddCredentialsAsync(connection, password);

            ActiveConnection = connection;

            return await Task.FromResult(true);
        }
    }
}
